/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CSSNamedImageValue_h
#define CSSNamedImageValue_h

#include "CSSImageGeneratorValue.h"

#include "Image.h"

namespace WebCore {

class Document;

class CSSNamedImageValue : public CSSImageGeneratorValue {
public:
    static Ref<CSSNamedImageValue> create(const String& name)
    {
        return adoptRef(*new CSSNamedImageValue(name));
    }

    String customCSSText() const;

    bool isFixedSize() const { return false; }
    bool isPending() const { return false; }

    RefPtr<Image> image(RenderElement*, const FloatSize&);

    bool equals(const CSSNamedImageValue&) const;

private:
    explicit CSSNamedImageValue(const String& name)
        : CSSImageGeneratorValue(NamedImageClass)
        , m_name(name)
    {
    }

    String m_name;
    RefPtr<Image> m_generatedImage;
};

} // namespace WebCore

SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSNamedImageValue, isNamedImageValue())

#endif // CSSNamedImageValue_h
